# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags for the compute packet mirroring commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.compute import completers as compute_completers
from googlecloudsdk.command_lib.compute import flags as compute_flags


class PacketMirroringCompleter(compute_completers.ListCommandCompleter):

  def __init__(self, **kwargs):
    super(PacketMirroringCompleter, self).__init__(
        collection='compute.packetMirrorings',
        list_command='alpha compute packet-mirrorings list --uri',
        **kwargs)


def PacketMirroringArgument(required=True, plural=False):
  return compute_flags.ResourceArgument(
      resource_name='packet mirroring',
      completer=PacketMirroringCompleter,
      plural=plural,
      custom_plural='packet mirrorings',
      required=required,
      regional_collection='compute.packetMirrorings')


def AddCreateArgs(parser):
  """Adds flags for creating packet mirroring resources."""
  _AddArgs(parser)


def AddUpdateArgs(parser):
  """Adds flags for updating packet mirroring resources."""
  _AddArgs(parser, is_for_update=True)


def _AddArgs(parser, is_for_update=False):
  """Adds args for create/update subcommands."""
  # Network cannot be updated.
  if not is_for_update:
    _AddNetworkArg(parser)

  _AddCollectorIlbArg(parser, is_for_update)

  parser.add_argument(
      '--description',
      help='Optional, textual description for the packet mirroring.')

  parser.add_argument(
      '--enable',
      action='store_true',
      help="""Enable or disable the packet-mirroring.""",
      default=True if not is_for_update else None)

  _AddMirroredInstancesArg(parser, is_for_update)
  _AddMirroredTagsArg(parser, is_for_update)
  _AddMirroredSubnetsArg(parser, is_for_update)

  _AddFilterCidrRangesArg(parser, is_for_update)
  _AddFilterProtocolsArg(parser, is_for_update)
  _AddFilterDirectionArg(parser)


def _AddNetworkArg(parser):
  parser.add_argument(
      '--network',
      required=True,
      help="""\
        Network for this packet mirroring.
        Only the packets in this network will be mirrored. It is mandatory
        that all mirrored VMs have a network interface controller (NIC) in
        the given network. All mirrored subnetworks should belong to the
        given network.

        You can provide this as the full URL to the network, partial URL,
        or name.
        For example, the following are valid values:
          * https://compute.googleapis.com/compute/v1/projects/myproject/
            global/networks/network-1
          * projects/myproject/global/networks/network-1
          * network-1
        """)


def _AddCollectorIlbArg(parser, is_for_update=False):
  parser.add_argument(
      '--collector-ilb',
      required=not is_for_update,
      help="""\
      Forwarding rule configured as collector. This must be a regional
      forwarding rule (in the same region) with load balancing scheme INTERNAL
      and isMirroringCollector set to true.

      You can provide this as the full URL to the forwarding rule, partial URL,
      or name.
      For example, the following are valid values:
        * https://compute.googleapis.com/compute/v1/projects/myproject/
          regions/us-central1/forwardingRules/fr-1
        * projects/myproject/regions/us-central1/forwardingRules/fr-1
        * fr-1
      """)


def _AddMirroredInstancesArg(parser, is_for_update=False):
  """Adds args to specify mirrored instances."""
  if is_for_update:
    instances = parser.add_mutually_exclusive_group(help="""\
      Update the mirrored instances of this packet mirroring.
      """)
    instances.add_argument(
        '--add-mirrored-instances',
        type=arg_parsers.ArgList(),
        metavar='INSTANCE',
        help='List of instances to add to the packet mirroring.',
    )
    instances.add_argument(
        '--remove-mirrored-instances',
        type=arg_parsers.ArgList(),
        metavar='INSTANCE',
        help='List of instances to remove from the packet mirroring.'
    )
    instances.add_argument(
        '--set-mirrored-instances',
        type=arg_parsers.ArgList(),
        metavar='INSTANCE',
        help='List of instances to be mirrored on the packet mirroring.'
    )
    instances.add_argument(
        '--clear-mirrored-instances',
        action='store_true',
        default=None,
        help=('If specified, clear the existing instances '
              'from the packet mirroring.'))
  else:
    parser.add_argument(
        '--mirrored-instances',
        type=arg_parsers.ArgList(),
        metavar='INSTANCE',
        help="""\
        List of instances to be mirrored.
        You can provide this as the full or valid partial URL to the instance.
        For example, the following are valid values:
          * https://compute.googleapis.com/compute/v1/projects/myproject/
            zones/us-central1-a/instances/instance-
          * projects/myproject/zones/us-central1-a/instances/instance-1
        """)


def _AddMirroredSubnetsArg(parser, is_for_update=False):
  """Adds args to specify mirrored subnets."""
  if is_for_update:
    subnets = parser.add_mutually_exclusive_group(help="""\
      Update the mirrored subnets of this packet mirroring.
      """)
    subnets.add_argument(
        '--add-mirrored-subnets',
        type=arg_parsers.ArgList(),
        metavar='SUBNET',
        help='List of subnets to add to the packet mirroring.',
    )
    subnets.add_argument(
        '--remove-mirrored-subnets',
        type=arg_parsers.ArgList(),
        metavar='SUBNET',
        help='List of subnets to remove from the packet mirroring.'
    )
    subnets.add_argument(
        '--set-mirrored-subnets',
        type=arg_parsers.ArgList(),
        metavar='SUBNET',
        help='List of subnets to be mirrored on the packet mirroring.'
    )
    subnets.add_argument(
        '--clear-mirrored-subnets',
        action='store_true',
        default=None,
        help=('If specified, clear the existing subnets '
              'from the packet mirroring.'))
  else:
    parser.add_argument(
        '--mirrored-subnets',
        type=arg_parsers.ArgList(),
        metavar='SUBNET',
        help="""\
        List of subnets to be mirrored.
        You can provide this as the full URL to the subnet, partial URL, or
        name.
        For example, the following are valid values:
          * https://compute.googleapis.com/compute/v1/projects/myproject/
            regions/us-central1/subnetworks/subnet-1
          * projects/myproject/regions/us-central1/subnetworks/subnet-1
          * subnet-1
        """)


def _AddMirroredTagsArg(parser, is_for_update=False):
  """Adds args to specify mirrored tags."""
  if is_for_update:
    tags = parser.add_mutually_exclusive_group(help="""\
      Update the mirrored tags of this packet mirroring.

      To read more about configuring network tags, read this guide:
      https://cloud.google.com/vpc/docs/add-remove-network-tags

      The virtual machines with the provided tags must live
      in zones contained in the same region as this packet mirroring.
      """)
    tags.add_argument(
        '--add-mirrored-tags',
        type=arg_parsers.ArgList(),
        metavar='TAG',
        help='List of tags to add to the packet mirroring.',
    )
    tags.add_argument(
        '--remove-mirrored-tags',
        type=arg_parsers.ArgList(),
        metavar='TAG',
        help='List of tags to remove from the packet mirroring.'
    )
    tags.add_argument(
        '--set-mirrored-tags',
        type=arg_parsers.ArgList(),
        metavar='TAG',
        help='List of tags to be mirrored on the packet mirroring.'
    )
    tags.add_argument(
        '--clear-mirrored-tags', action='store_true', default=None,
        help='If specified, clear the existing tags from the packet mirroring.')
  else:
    parser.add_argument(
        '--mirrored-tags',
        type=arg_parsers.ArgList(),
        metavar='TAG',
        help="""\
        List of virtual machine instance tags to be mirrored.

        To read more about configuring network tags, read this guide:
        https://cloud.google.com/vpc/docs/add-remove-network-tags

        The virtual machines with the provided tags must live
        in zones contained in the same region as this packet mirroring.
        """)


def _AddFilterCidrRangesArg(parser, is_for_update=False):
  """Adds args to specify filter CIDR ranges."""
  if is_for_update:
    cidr_ranges = parser.add_mutually_exclusive_group(
        help='Update the filter CIDR ranges of this packet mirroring.')
    cidr_ranges.add_argument(
        '--add-filter-cidr-ranges',
        type=arg_parsers.ArgList(),
        metavar='CIDR_RANGE',
        help='List of filter CIDR ranges to add to the packet mirroring.'
    )
    cidr_ranges.add_argument(
        '--remove-filter-cidr-ranges',
        type=arg_parsers.ArgList(),
        metavar='CIDR_RANGE',
        help='List of filter CIDR ranges to remove from the packet mirroring.'
    )
    cidr_ranges.add_argument(
        '--set-filter-cidr-ranges',
        type=arg_parsers.ArgList(),
        metavar='CIDR_RANGE',
        help="""\
        List of filter CIDR ranges to be mirrored on the packet mirroring.
        """)
    cidr_ranges.add_argument(
        '--clear-filter-cidr-ranges',
        action='store_true',
        default=None,
        help="""\
        If specified, clear the existing filter CIDR ranges from the packet
        mirroring.
        """
    )
  else:
    parser.add_argument(
        '--filter-cidr-ranges',
        type=arg_parsers.ArgList(),
        metavar='CIDR_RANGE',
        help="""\
        List of IP CIDR ranges that apply as filters on the source or
        destination IP in the IP header for packet mirroring traffic. All
        traffic between the VM and the IPs listed here will be mirrored using
        this configuration. This can be a Public IP as well.
        If unspecified, the config applies to all traffic.
        """
    )


def _AddFilterProtocolsArg(parser, is_for_update=False):
  """Adds args to specify filter IP protocols."""
  if is_for_update:
    protocols = parser.add_mutually_exclusive_group(
        help='Update the filter protocols of this packet mirroring.')
    protocols.add_argument(
        '--add-filter-protocols',
        type=arg_parsers.ArgList(element_type=str),
        metavar='PROTOCOL',
        help="""\
        List of filter IP protocols to add to the packet mirroring.
        PROTOCOL can be one of tcp, udp, icmp, esp, ah, ipip, sctp, or an IANA
        protocol number.
        """)
    protocols.add_argument(
        '--remove-filter-protocols',
        type=arg_parsers.ArgList(element_type=str),
        metavar='PROTOCOL',
        help="""\
        List of filter IP protocols to remove from the packet mirroring.
        PROTOCOL can be one of tcp, udp, icmp, esp, ah, ipip, sctp, or an IANA
        protocol number.
        """)
    protocols.add_argument(
        '--set-filter-protocols',
        type=arg_parsers.ArgList(element_type=str),
        metavar='PROTOCOL',
        help="""\
        List of filter IP protocols to be mirrored on the packet mirroring.
        PROTOCOL can be one of tcp, udp, icmp, esp, ah, ipip, sctp, or an IANA
        protocol number.
        """)
    protocols.add_argument(
        '--clear-filter-protocols',
        action='store_true',
        default=None,
        help="""\
        If specified, clear the existing filter IP protocols from the packet
        mirroring.
        """)
  else:
    parser.add_argument(
        '--filter-protocols',
        type=arg_parsers.ArgList(element_type=str),
        metavar='PROTOCOL',
        help="""\
        List of IP protocols that apply as filters for packet mirroring traffic.
        If unspecified, the packet mirroring applies to all traffic.
        PROTOCOL can be one of tcp, udp, icmp, esp, ah, ipip, sctp, or an IANA
        protocol number.
        """,
    )


def _AddFilterDirectionArg(parser):
  """Adds args to specify filter direction."""
  parser.add_argument(
      '--filter-direction',
      choices=['both', 'egress', 'ingress'],
      metavar='DIRECTION',
      help="""\
        - For `ingress`, only ingress traffic is mirrored.
        - For `egress`, only egress traffic is mirrored.
        - For `both` (default), both directions are mirrored.
        """,
  )



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for the cloud deploy delivery pipeline resource."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.clouddeploy import delivery_pipeline
from googlecloudsdk.command_lib.deploy import exceptions as cd_exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import resources

_PIPELINES_WITH_GIVEN_TARGET_FILTER_TEMPLATE = (
    'serialPipeline.stages.targetId:"{}"')


def ListDeliveryPipelinesWithTarget(target_ref, location_ref):
  """Lists the delivery pipelines associated with the specified target.

  The returned list is sorted by the delivery pipeline's create time.
  Args:
    target_ref: protorpc.messages.Message, target object.
    location_ref: protorpc.messages.Message, location object.

  Returns:
    a sorted list of delivery pipelines.
  """
  filter_str = _PIPELINES_WITH_GIVEN_TARGET_FILTER_TEMPLATE.format(
      target_ref.Name())

  pipelines = delivery_pipeline.DeliveryPipelinesClient().List(
      location=location_ref.RelativeName(),
      filter_str=filter_str,
      page_size=0,
  )

  # ListDeliveryPipelines does not support orderBy=createTime field
  # so sort result to get the same result you would get by using
  # orderBy = 'createTime desc'
  return sorted(
      pipelines, key=lambda pipeline: pipeline.createTime, reverse=True)


def PipelineToPipelineRef(pipeline):
  pipeline_ref = resources.REGISTRY.ParseRelativeName(
      pipeline.name,
      collection='clouddeploy.projects.locations.deliveryPipelines')
  return pipeline_ref


def GetPipeline(pipeline_name):
  """Gets the delivery pipeline and returns the value of its suspended field.

  Args:
    pipeline_name: str, the canonical resource name of the delivery pipeline

  Returns:
    The pipeline object
  Raises:
    apitools.base.py.HttpError
  """
  try:
    pipeline_obj = delivery_pipeline.DeliveryPipelinesClient().Get(
        pipeline_name)
    return pipeline_obj
  except apitools_exceptions.HttpError as error:
    log.debug('Failed to get pipeline {}: {}'.format(pipeline_name,
                                                     error.content))
    log.status.Print('Unable to get delivery pipeline {}'.format(pipeline_name))
    raise error


def ThrowIfPipelineSuspended(pipeline_obj, suspended_pipeline_msg):
  """Checks if the delivery pipeline associated with the release is suspended.

  Args:
    pipeline_obj: protorpc.messages.Message, delivery pipeline object.
    suspended_pipeline_msg: str, error msg to show the user if the pipeline is
      suspended.

  Raises:
    googlecloudsdk.command_lib.deploy.PipelineSuspendedError if the pipeline is
    suspended
  """
  if pipeline_obj.suspended:
    raise cd_exceptions.PipelineSuspendedError(
        pipeline_name=pipeline_obj.name,
        failed_activity_msg=suspended_pipeline_msg)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Script for reporting gcloud metrics."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import os
import pickle
import sys

# This file is unique because it is executed as a separate process on metrics
# teardown. Because of this, googlecloudsdk.core is on the Python Path (because
# the directory of the main script is always first on the Path). This happens
# not to matter on Python2, but on Python3, there is a module named http.client
# that is used by httplib2. When core is on the path, our http.py shadows this
# module and the import fails. On Python2, this module doesn't exist so we just
# hadn't hit this issue. Here, we remove the script's directory from the Path
# because we never want to import anything from here as an absolute import (on
# either Python2 or Python3).
sys.path.pop(0)

# pylint: disable=g-import-not-at-top
from googlecloudsdk.core import argv_utils
from googlecloudsdk.core.util import files

try:
  from googlecloudsdk.core import requests
except ImportError:
  # Do nothing if we can't import the lib.
  sys.exit(0)

# If outgoing packets are getting dropped, httplib2 will not respond
# indefinitely while waiting for a response.
TIMEOUT_IN_SEC = 10


def ReportMetrics(metrics_file_path):
  """Sends the specified anonymous usage event to the given analytics endpoint.

  Args:
      metrics_file_path: str, File with pickled metrics (list of tuples).
  """
  with files.BinaryFileReader(metrics_file_path) as metrics_file:
    metrics = pickle.load(metrics_file)
  os.remove(metrics_file_path)

  session = requests.Session()

  for metric in metrics:
    session.request(metric[1], metric[0], data=metric[2], headers=metric[3],
                    timeout=TIMEOUT_IN_SEC)

if __name__ == '__main__':
  try:
    ReportMetrics(argv_utils.GetDecodedArgv()[1])
  # pylint: disable=bare-except, Never fail or output a stacktrace here.
  except:
    pass



# -*- coding: utf-8 -*-
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Additional help about the shim for running gcloud storage."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from gslib.help_provider import HelpProvider

_DETAILED_HELP_TEXT = ("""
<B>OVERVIEW</B>
Cloud SDK includes a new CLI, gcloud storage, that can be considerably faster
than gsutil when performing uploads and downloads with less parameter tweaking.
This new CLI has a syntax and command structure that is familiar to gsutil users
but is fundamentally different in many important ways. To ease transition to
this new CLI, gsutil provides a shim that translates your gsutil commands to
gcloud storage commands if an equivalent exists, and falls back to gsutil's
usual behavior if an equivalent does not exist.


<B>TO ENABLE</B>
Set the use_gcloud_storage field in the .boto config file under the GSUtil
section.

  [GSUtil]
  use_gcloud_storage=True

You can also set the flag for individual commands using the top-level -o flag.

  gsutil -o "GSUtil:use_gcloud_storage=True" -m cp -p file gs://bucket/obj


<B>AVAILABLE COMMANDS</B>
The gcloud storage CLI only supports a subset of gsutil commands. What follows
is a list of commands supported by the shim with any differences in behavior
noted.

cat:
- Prints object data for a second object even if the first object is invalid.
compose:
- Works as expected.
cp:
- Copies a second object even if the first object is invalid.
- Does not support file to file copies.
- Supports copying objects cloud-to-cloud with trailing slashes in the name.
- The all-version flag (-A) silently enables sequential execution rather than
  raising an error.
defstorageclass:
- Works as expected.
kms:
- The authorize subcommand returns informational messages in a different format.
- The encryption subcommand returns informational messages in a different
  format.
logging:
- The get subcommand has different JSON spacing and doesn't print an
  informational message if no configuration is found.
ls:
- Works as expected.
mb:
- Works as expected.
mv:
- See notes on cp.
notification:
- The list subcommand prints configuration information as YAML.
- The delete subcommand offers progress tracking and parallelization.
pap:
- Works as expected.
rb:
- Works as expected.
requesterpays:
- Works as expected.
rewrite:
- The -k flag does not throw an error if called without a new key. In both the
  shim and unshimmed cases, the old key is maintained.
rm:
- $folder$ delete markers are not supported.
setmeta:
- Does not throw an error if no headers are changed.
stat:
- Uses a list call rather than a get call, which is potentially not strongly
  consistent (this will be fixed in a later version).
- Includes a field “Storage class update time:” which may throw off tabbing.
versioning:
- Works as expected.
web:
- The get subcommand has different JSON spacing and doesn't print an
  informational message if no configuration is found.



<B>BOTO CONFIGURATION</B>
Configuration found in the boto file is mapped 1:1 to gcloud environment
variables where appropriate.

[Credentials]
- aws_access_key_id: AWS_ACCESS_KEY_ID
- aws_secret_access_key: AWS_SECRET_ACCESS_KEY
- use_client_certificate: CLOUDSDK_CONTEXT_AWARE_USE_CLIENT_CERTIFICATE

[Boto]
- proxy: CLOUDSDK_PROXY_ADDRESS
- proxy_type: CLOUDSDK_PROXY_TYPE
- proxy_port: CLOUDSDK_PROXY_PORT
- proxy_user: CLOUDSDK_PROXY_USERNAME
- proxy_pass: CLOUDSDK_PROXY_PASSWORD
- proxy_rdns: CLOUDSDK_PROXY_RDNS
- http_socket_timeout: CLOUDSDK_CORE_HTTP_TIMEOUT
- ca_certificates_file: CLOUDSDK_CORE_CUSTOM_CA_CERTS_FILE
- max_retry_delay: CLOUDSDK_STORAGE_BASE_RETRY_DELAY
- num_retries: CLOUDSDK_STORAGE_MAX_RETRIES

[GSUtil]
- check_hashes: CLOUDSDK_STORAGE_CHECK_HASHES
- default_project_id: CLOUDSDK_CORE_PROJECT
- disable_analytics_prompt: CLOUDSDK_CORE_DISABLE_USAGE_REPORTING
- use_magicfile: CLOUDSDK_STORAGE_USE_MAGICFILE
- parallel_composite_upload_threshold: CLOUDSDK_STORAGE_PARALLEL_COMPOSITE_UPLOAD_THRESHOLD
- resumable_threshold: CLOUDSDK_STORAGE_RESUMABLE_THRESHOLD

[OAuth2]
- client_id: CLOUDSDK_AUTH_CLIENT_ID
- client_secret: CLOUDSDK_AUTH_CLIENT_SECRET
- provider_authorization_uri: CLOUDSDK_AUTH_AUTH_HOST
- provider_token_uri: CLOUDSDK_AUTH_TOKEN_HOST


<B>GENERAL COMPATIBILITY NOTES</B>
- Due to its compatibility across all major platforms, multiprocessing is
  enabled for all commands by default (equivalent to the -m option always being
  included in gsutil).
- A sequence of asterisks greater than 2 (i.e. ***) are always treated as a
  single asterisk.
- Unlike gsutil, gcloud is not designed to be used in parallel invocations, and
  doing so (i.e. running the shim from 2 terminals at once) can lead to
  unpredictable behavior.
- Assuming a bucket contains an object gs://bucket/nested/foo.txt, gsutil's
  wildcard iterator will match foo.txt given a URL like gs://bucket/*/nested/.
  The shim will not match foo.txt given the same URL.
- This will be updated as new commands are supported by both gcloud storage and
  the shim.

""")


class CommandOptions(HelpProvider):
  """Additional help about the shim for running gcloud storage."""

  # Help specification. See help_provider.py for documentation.
  help_spec = HelpProvider.HelpSpec(
      help_name='shim',
      help_name_aliases=['gcloudstorage', 'gcloud storage'],
      help_type='additional_help',
      help_one_line_summary='Shim for running gcloud storage',
      help_text=_DETAILED_HELP_TEXT,
      subcommand_help_text={},
  )



"""Generated message classes for apikeys version v2.

Manages the API keys associated with developer projects.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'apikeys'


class ApikeysKeysLookupKeyRequest(_messages.Message):
  r"""A ApikeysKeysLookupKeyRequest object.

  Fields:
    keyString: Required. Finds the project that owns the key string value.
  """

  keyString = _messages.StringField(1)


class ApikeysOperationsGetRequest(_messages.Message):
  r"""A ApikeysOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class ApikeysProjectsLocationsKeysCreateRequest(_messages.Message):
  r"""A ApikeysProjectsLocationsKeysCreateRequest object.

  Fields:
    keyId: User specified key id (optional). If specified, it will become the
      final component of the key resource name. The id must be unique within
      the project, must conform with RFC-1034, is restricted to lower-cased
      letters, and has a maximum length of 63 characters. In another word, the
      id must match the regular expression: `[a-z]([a-z0-9-]{0,61}[a-z0-9])?`.
      The id must NOT be a UUID-like string.
    parent: Required. The project in which the API key is created.
    v2Key: A V2Key resource to be passed as the request body.
  """

  keyId = _messages.StringField(1)
  parent = _messages.StringField(2, required=True)
  v2Key = _messages.MessageField('V2Key', 3)


class ApikeysProjectsLocationsKeysDeleteRequest(_messages.Message):
  r"""A ApikeysProjectsLocationsKeysDeleteRequest object.

  Fields:
    etag: Optional. The etag known to the client for the expected state of the
      key. This is to be used for optimistic concurrency.
    name: Required. The resource name of the API key to be deleted.
  """

  etag = _messages.StringField(1)
  name = _messages.StringField(2, required=True)


class ApikeysProjectsLocationsKeysGetKeyStringRequest(_messages.Message):
  r"""A ApikeysProjectsLocationsKeysGetKeyStringRequest object.

  Fields:
    name: Required. The resource name of the API key to be retrieved.
  """

  name = _messages.StringField(1, required=True)


class ApikeysProjectsLocationsKeysGetRequest(_messages.Message):
  r"""A ApikeysProjectsLocationsKeysGetRequest object.

  Fields:
    name: Required. The resource name of the API key to get.
  """

  name = _messages.StringField(1, required=True)


class ApikeysProjectsLocationsKeysListRequest(_messages.Message):
  r"""A ApikeysProjectsLocationsKeysListRequest object.

  Fields:
    pageSize: Optional. Specifies the maximum number of results to be returned
      at a time.
    pageToken: Optional. Requests a specific page of results.
    parent: Required. Lists all API keys associated with this project.
    showDeleted: Optional. Indicate that keys deleted in the past 30 days
      should also be returned.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  showDeleted = _messages.BooleanField(4)


class ApikeysProjectsLocationsKeysPatchRequest(_messages.Message):
  r"""A ApikeysProjectsLocationsKeysPatchRequest object.

  Fields:
    name: Output only. The resource name of the key. The `name` has the form:
      `projects//locations/global/keys/`. For example: `projects/123456867718/
      locations/global/keys/b7ff1f9f-8275-410a-94dd-3855ee9b5dd2` NOTE: Key is
      a global resource; hence the only supported value for location is
      `global`.
    updateMask: The field mask specifies which fields to be updated as part of
      this request. All other fields are ignored. Mutable fields are:
      `display_name`,`restrictions` and `annotations`. If an update mask is
      not provided, the service treats it as an implied mask equivalent to all
      allowed fields that are set on the wire. If the field mask has a special
      value "*", the service treats it equivalent to replace all allowed
      mutable fields.
    v2Key: A V2Key resource to be passed as the request body.
  """

  name = _messages.StringField(1, required=True)
  updateMask = _messages.StringField(2)
  v2Key = _messages.MessageField('V2Key', 3)


class ApikeysProjectsLocationsKeysUndeleteRequest(_messages.Message):
  r"""A ApikeysProjectsLocationsKeysUndeleteRequest object.

  Fields:
    name: Required. The resource name of the API key to be undeleted.
    v2UndeleteKeyRequest: A V2UndeleteKeyRequest resource to be passed as the
      request body.
  """

  name = _messages.StringField(1, required=True)
  v2UndeleteKeyRequest = _messages.MessageField('V2UndeleteKeyRequest', 2)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success. If
      the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal response of the operation in case of success. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success. If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class V2AndroidApplication(_messages.Message):
  r"""Identifier of an Android application for key use.

  Fields:
    packageName: The package name of the application.
    sha1Fingerprint: The SHA1 fingerprint of the application. For example,
      both sha1 formats are acceptable :
      DA:39:A3:EE:5E:6B:4B:0D:32:55:BF:EF:95:60:18:90:AF:D8:07:09 or
      DA39A3EE5E6B4B0D3255BFEF95601890AFD80709. Output format is the latter.
  """

  packageName = _messages.StringField(1)
  sha1Fingerprint = _messages.StringField(2)


class V2AndroidKeyRestrictions(_messages.Message):
  r"""The Android apps that are allowed to use the key.

  Fields:
    allowedApplications: A list of Android applications that are allowed to
      make API calls with this key.
  """

  allowedApplications = _messages.MessageField('V2AndroidApplication', 1, repeated=True)


class V2ApiTarget(_messages.Message):
  r"""A restriction for a specific service and optionally one or multiple
  specific methods. Both fields are case insensitive.

  Fields:
    methods: Optional. List of one or more methods that can be called. If
      empty, all methods for the service are allowed. A wildcard (*) can be
      used as the last symbol. Valid examples:
      `google.cloud.translate.v2.TranslateService.GetSupportedLanguage`
      `TranslateText` `Get*` `translate.googleapis.com.Get*`
    service: The service for this restriction. It should be the canonical
      service name, for example: `translate.googleapis.com`. You can use
      [`gcloud services list`](/sdk/gcloud/reference/services/list) to get a
      list of services that are enabled in the project.
  """

  methods = _messages.StringField(1, repeated=True)
  service = _messages.StringField(2)


class V2BrowserKeyRestrictions(_messages.Message):
  r"""The HTTP referrers (websites) that are allowed to use the key.

  Fields:
    allowedReferrers: A list of regular expressions for the referrer URLs that
      are allowed to make API calls with this key.
  """

  allowedReferrers = _messages.StringField(1, repeated=True)


class V2GetKeyStringResponse(_messages.Message):
  r"""Response message for `GetKeyString` method.

  Fields:
    keyString: An encrypted and signed value of the key.
  """

  keyString = _messages.StringField(1)


class V2IosKeyRestrictions(_messages.Message):
  r"""The iOS apps that are allowed to use the key.

  Fields:
    allowedBundleIds: A list of bundle IDs that are allowed when making API
      calls with this key.
  """

  allowedBundleIds = _messages.StringField(1, repeated=True)


class V2Key(_messages.Message):
  r"""The representation of a key managed by the API Keys API.

  Messages:
    AnnotationsValue: Annotations is an unstructured key-value map stored with
      a policy that may be set by external tools to store and retrieve
      arbitrary metadata. They are not queryable and should be preserved when
      modifying objects.

  Fields:
    annotations: Annotations is an unstructured key-value map stored with a
      policy that may be set by external tools to store and retrieve arbitrary
      metadata. They are not queryable and should be preserved when modifying
      objects.
    createTime: Output only. A timestamp identifying the time this key was
      originally created.
    deleteTime: Output only. A timestamp when this key was deleted. If the
      resource is not deleted, this must be empty.
    displayName: Human-readable display name of this key that you can modify.
      The maximum length is 63 characters.
    etag: Output only. A checksum computed by the server based on the current
      value of the Key resource. This may be sent on update and delete
      requests to ensure the client has an up-to-date value before proceeding.
      See https://google.aip.dev/154.
    keyString: Output only. An encrypted and signed value held by this key.
      This field can be accessed only through the `GetKeyString` method.
    name: Output only. The resource name of the key. The `name` has the form:
      `projects//locations/global/keys/`. For example: `projects/123456867718/
      locations/global/keys/b7ff1f9f-8275-410a-94dd-3855ee9b5dd2` NOTE: Key is
      a global resource; hence the only supported value for location is
      `global`.
    restrictions: Key restrictions.
    uid: Output only. Unique id in UUID4 format.
    updateTime: Output only. A timestamp identifying the time this key was
      last updated.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AnnotationsValue(_messages.Message):
    r"""Annotations is an unstructured key-value map stored with a policy that
    may be set by external tools to store and retrieve arbitrary metadata.
    They are not queryable and should be preserved when modifying objects.

    Messages:
      AdditionalProperty: An additional property for a AnnotationsValue
        object.

    Fields:
      additionalProperties: Additional properties of type AnnotationsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AnnotationsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  annotations = _messages.MessageField('AnnotationsValue', 1)
  createTime = _messages.StringField(2)
  deleteTime = _messages.StringField(3)
  displayName = _messages.StringField(4)
  etag = _messages.StringField(5)
  keyString = _messages.StringField(6)
  name = _messages.StringField(7)
  restrictions = _messages.MessageField('V2Restrictions', 8)
  uid = _messages.StringField(9)
  updateTime = _messages.StringField(10)


class V2ListKeysResponse(_messages.Message):
  r"""Response message for `ListKeys` method.

  Fields:
    keys: A list of API keys.
    nextPageToken: The pagination token for the next page of results.
  """

  keys = _messages.MessageField('V2Key', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class V2LookupKeyResponse(_messages.Message):
  r"""Response message for `LookupKey` method.

  Fields:
    name: The resource name of the API key. If the API key has been purged,
      resource name is empty.
    parent: The project that owns the key with the value specified in the
      request.
  """

  name = _messages.StringField(1)
  parent = _messages.StringField(2)


class V2Restrictions(_messages.Message):
  r"""Describes the restrictions on the key.

  Fields:
    androidKeyRestrictions: The Android apps that are allowed to use the key.
    apiTargets: A restriction for a specific service and optionally one or
      more specific methods. Requests are allowed if they match any of these
      restrictions. If no restrictions are specified, all targets are allowed.
    browserKeyRestrictions: The HTTP referrers (websites) that are allowed to
      use the key.
    iosKeyRestrictions: The iOS apps that are allowed to use the key.
    serverKeyRestrictions: The IP addresses of callers that are allowed to use
      the key.
  """

  androidKeyRestrictions = _messages.MessageField('V2AndroidKeyRestrictions', 1)
  apiTargets = _messages.MessageField('V2ApiTarget', 2, repeated=True)
  browserKeyRestrictions = _messages.MessageField('V2BrowserKeyRestrictions', 3)
  iosKeyRestrictions = _messages.MessageField('V2IosKeyRestrictions', 4)
  serverKeyRestrictions = _messages.MessageField('V2ServerKeyRestrictions', 5)


class V2ServerKeyRestrictions(_messages.Message):
  r"""The IP addresses of callers that are allowed to use the key.

  Fields:
    allowedIps: A list of the caller IP addresses that are allowed to make API
      calls with this key.
  """

  allowedIps = _messages.StringField(1, repeated=True)


class V2UndeleteKeyRequest(_messages.Message):
  r"""Request message for `UndeleteKey` method."""


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for updating env vars and other configuration info."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from googlecloudsdk.api_lib.run import k8s_object
from googlecloudsdk.api_lib.run import traffic
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.run import config_changes
from googlecloudsdk.command_lib.run import connection_context
from googlecloudsdk.command_lib.run import exceptions
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.command_lib.run import messages_util
from googlecloudsdk.command_lib.run import pretty_print
from googlecloudsdk.command_lib.run import resource_args
from googlecloudsdk.command_lib.run import resource_change_validators
from googlecloudsdk.command_lib.run import serverless_operations
from googlecloudsdk.command_lib.run import stages
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core.console import progress_tracker


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Update(base.Command):
  """Update Cloud Run environment variables and other configuration settings."""

  detailed_help = {
      'DESCRIPTION':
          """\
          {description}
          """,
      'EXAMPLES':
          """\
          To update one or more env vars:

              $ {command} myservice --update-env-vars=KEY1=VALUE1,KEY2=VALUE2
         """,
  }

  @staticmethod
  def CommonArgs(parser):
    # Flags specific to managed CR
    managed_group = flags.GetManagedArgGroup(parser)
    flags.AddBinAuthzPolicyFlags(managed_group)
    flags.AddBinAuthzBreakglassFlag(managed_group)
    flags.AddCloudSQLFlags(managed_group)
    flags.AddCpuThrottlingFlag(managed_group)
    flags.AddRevisionSuffixArg(managed_group)
    flags.AddVpcConnectorArgs(managed_group)
    flags.AddEgressSettingsFlag(managed_group)
    flags.AddCmekKeyFlag(managed_group)
    flags.AddCmekKeyRevocationActionTypeFlag(managed_group)

    # Flags specific to connecting to a cluster
    cluster_group = flags.GetClusterArgGroup(parser)
    flags.AddEndpointVisibilityEnum(cluster_group)
    flags.AddConfigMapsFlags(cluster_group)

    # Flags not specific to any platform
    service_presentation = presentation_specs.ResourcePresentationSpec(
        'SERVICE',
        resource_args.GetServiceResourceSpec(prompt=True),
        'Service to update the configuration of.',
        required=True,
        prefixes=False)
    flags.AddMutexEnvVarsFlags(parser)
    flags.AddMemoryFlag(parser)
    flags.AddConcurrencyFlag(parser)
    flags.AddTimeoutFlag(parser)
    flags.AddAsyncFlag(parser)
    flags.AddLabelsFlags(parser)
    flags.AddGeneralAnnotationFlags(parser)
    flags.AddMinInstancesFlag(parser)
    flags.AddMaxInstancesFlag(parser)
    flags.AddCommandFlag(parser)
    flags.AddArgsFlag(parser)
    flags.AddPortFlag(parser)
    flags.AddCpuFlag(parser)
    flags.AddNoTrafficFlag(parser)
    flags.AddDeployTagFlag(parser)
    flags.AddServiceAccountFlag(parser)
    flags.AddImageArg(parser, required=False)
    flags.AddClientNameAndVersionFlags(parser)
    flags.AddIngressFlag(parser)
    flags.AddHttp2Flag(parser)
    flags.AddSecretsFlags(parser)
    concept_parsers.ConceptParser([service_presentation]).AddToParser(parser)
    # No output by default, can be overridden by --format
    parser.display_info.AddFormat('none')

  @staticmethod
  def Args(parser):
    Update.CommonArgs(parser)

  def Run(self, args):
    """Update the service resource.

       Different from `deploy` in that it can only update the service spec but
       no IAM or Cloud build changes.

    Args:
      args: Args!
    Returns:
      googlecloudsdk.api_lib.run.Service, the updated service
    """
    changes = flags.GetServiceConfigurationChanges(args)
    if not changes or (len(changes) == 1 and isinstance(
        changes[0], config_changes.SetClientNameAndVersionAnnotationChange)):
      raise exceptions.NoConfigurationChangeError(
          'No configuration change requested. '
          'Did you mean to include the flags `--update-env-vars`, '
          '`--memory`, `--concurrency`, `--timeout`, `--connectivity`, '
          '`--image`?')
    changes.insert(
        0,
        config_changes.DeleteAnnotationChange(
            k8s_object.BINAUTHZ_BREAKGLASS_ANNOTATION))
    changes.append(
        config_changes.SetLaunchStageAnnotationChange(self.ReleaseTrack()))

    conn_context = connection_context.GetConnectionContext(
        args, flags.Product.RUN, self.ReleaseTrack())
    service_ref = args.CONCEPTS.service.Parse()
    flags.ValidateResource(service_ref)

    with serverless_operations.Connect(conn_context) as client:
      service = client.GetService(service_ref)
      resource_change_validators.ValidateClearVpcConnector(service, args)
      has_latest = (
          service is None or
          traffic.LATEST_REVISION_KEY in service.spec_traffic)
      deployment_stages = stages.ServiceStages(
          include_iam_policy_set=False, include_route=has_latest)
      with progress_tracker.StagedProgressTracker(
          'Deploying...',
          deployment_stages,
          failure_message='Deployment failed',
          suppress_output=args.async_) as tracker:
        service = client.ReleaseService(
            service_ref, changes, tracker, asyn=args.async_, prefetch=service)

      if args.async_:
        pretty_print.Success('Service [{{bold}}{serv}{{reset}}] is deploying '
                             'asynchronously.'.format(serv=service.name))
      else:
        service = client.GetService(service_ref)
        pretty_print.Success(
            messages_util.GetSuccessMessageForSynchronousDeploy(service))
      return service


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class BetaUpdate(Update):
  """Update Cloud Run environment variables and other configuration settings."""

  @staticmethod
  def Args(parser):
    Update.CommonArgs(parser)

    # Flags specific to managed CR
    managed_group = flags.GetManagedArgGroup(parser)
    flags.AddSandboxArg(managed_group)
    flags.AddSessionAffinityFlag(managed_group)
    flags.AddStartupCpuBoostFlag(managed_group)
    flags.AddEncryptionKeyShutdownHoursFlag(managed_group)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class AlphaUpdate(Update):
  """Update Cloud Run environment variables and other configuration settings."""

  @staticmethod
  def Args(parser):
    Update.CommonArgs(parser)

    # Flags specific to managed CR
    managed_group = flags.GetManagedArgGroup(parser)
    flags.AddSandboxArg(managed_group)
    flags.AddCustomAudiencesFlag(managed_group)
    flags.AddSessionAffinityFlag(managed_group)
    flags.AddStartupCpuBoostFlag(managed_group)
    flags.AddEncryptionKeyShutdownHoursFlag(managed_group)
    flags.AddVpcNetworkFlags(managed_group)
    flags.AddVpcSubnetFlags(managed_group)
    flags.AddVpcNetworkTagsFlags(managed_group)


AlphaUpdate.__doc__ = Update.__doc__



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for Vertex AI hyperparameter tuning jobs commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.core import resources

HPTUNING_JOB_COLLECTION = 'aiplatform.projects.locations.hyperparameterTuningJobs'


def ParseJobName(name):
  """Parses the id from a full hyperparameter tuning job name."""
  return resources.REGISTRY.Parse(
      name, collection=HPTUNING_JOB_COLLECTION).Name()


def OutputCommandVersion(release_track):
  if release_track == base.ReleaseTrack.GA:
    return ''
  elif release_track == base.ReleaseTrack.BETA:
    return ' beta'
  else:
    return ' alpha'



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Base classes for abstracting away common logic."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import enum

from googlecloudsdk.api_lib.util import apis

API_NAME = 'recommender'

RECOMMENDER_MESSAGE_PREFIX = {
    'v1': 'GoogleCloudRecommenderV1',
    'v1beta1': 'GoogleCloudRecommenderV1beta1',
    'v1alpha2': 'GoogleCloudRecommenderV1alpha2'
}


class EntityType(enum.Enum):
  """Cloud Entity Types."""
  ORGANIZATION = 1
  FOLDER = 2
  PROJECT = 3
  BILLING_ACCOUNT = 4


class ClientBase(object):
  """Base client class for all versions."""

  def __init__(self, api_version):
    self._client = apis.GetClientInstance(API_NAME, api_version)
    self._api_version = api_version
    self._messages = self._client.MESSAGES_MODULE
    self._message_prefix = RECOMMENDER_MESSAGE_PREFIX[api_version]

  def _GetMessage(self, message_name):
    """Returns the API messages class by name."""
    return getattr(self._messages, message_name, None)

  def _GetVersionedMessage(self, message_name):
    """Returns the versioned API messages class by name."""
    return self._GetMessage('{prefix}{name}'.format(
        prefix=self._message_prefix, name=message_name))
